@@ -3,7 +3,7 @@
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis-python
 #
-# Most of this work is copyright (C) 2013-2016 David R. MacIver
+# Most of this work is copyright (C) 2013-2017 David R. MacIver
 # (david@drmaciver.com), but it contains contributions by others. See
 # CONTRIBUTING.rst for a full list of people who may hold copyright, and
 # consult the git log if you need to determine who owns an individual
@@ -23,7 +23,6 @@ import warnings
 class HypothesisException(Exception):
 
     """Generic parent class for exceptions thrown by Hypothesis."""
-    pass
 
 
 class CleanupFailed(HypothesisException):
@@ -83,7 +82,6 @@ class NoExamples(HypothesisException):
     """Raised when example() is called on a strategy but we cannot find any
     examples after enough tries that we really should have been able to if this
     was ever going to work."""
-    pass
 
 
 class Unsatisfiable(HypothesisException):
@@ -115,7 +113,7 @@ class Flaky(HypothesisException):
            depends sensitively on where it's been called from.
         3. The function is timing sensitive and can fail or pass depending on
            how long it takes. Try breaking it up into smaller functions which
-           dont' do that and testing those instead.
+           don't do that and testing those instead.
 
     """
 
@@ -144,6 +142,17 @@ class InvalidArgument(HypothesisException, TypeError):
     some manner incorrect."""
 
 
+class ResolutionFailed(InvalidArgument):
+
+    """Hypothesis had to resolve a type to a strategy, but this failed.
+
+    Type inference is best-effort, so this only happens when an
+    annotation exists but could not be resolved for a required argument
+    to the target of ``builds()``, or where the user passed ``infer``.
+
+    """
+
+
 class InvalidState(HypothesisException):
 
     """The system is not in a state where you were allowed to do that."""
@@ -175,5 +184,5 @@ warnings.simplefilter('once', HypothesisDeprecationWarning)
 
 class Frozen(HypothesisException):
 
-    """Raised when a mutation method has been called on a TestData object after
-    freeze() has been called."""
+    """Raised when a mutation method has been called on a ConjectureData object
+    after freeze() has been called."""
