@@ -3,7 +3,7 @@
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis-python
 #
-# Most of this work is copyright (C) 2013-2015 David R. MacIver
+# Most of this work is copyright (C) 2013-2017 David R. MacIver
 # (david@drmaciver.com), but it contains contributions by others. See
 # CONTRIBUTING.rst for a full list of people who may hold copyright, and
 # consult the git log if you need to determine who owns an individual
@@ -26,7 +26,10 @@ import math
 import codecs
 import platform
 import importlib
+from gzip import GzipFile
+from base64 import b64decode, b64encode
 from decimal import Context, Decimal, Inexact
+from hashlib import sha1
 from collections import namedtuple
 
 try:
@@ -39,65 +42,22 @@ except ImportError:  # pragma: no cover
 PY2 = sys.version_info[0] == 2
 PY3 = sys.version_info[0] == 3
 PYPY = platform.python_implementation() == 'PyPy'
-PY26 = sys.version_info[:2] == (2, 6)
-NO_ARGSPEC = sys.version_info[:2] >= (3, 5)
-HAS_SIGNATURE = sys.version_info[:2] >= (3, 3)
 CAN_UNPACK_BYTE_ARRAY = sys.version_info[:3] >= (2, 7, 4)
 
 WINDOWS = platform.system() == 'Windows'
 
-if PY26:
-    from hypothesis import __version__ as thisversion
-    try:
-        from hypothesislegacysupport import __version__ as thatversion
-    except ImportError:
-        raise ImportError(
-            'Hypothesis is not supported on Python 2.6 without the '
-            'hypothesislegacysupport installed. Check that you have a '
-            'license to use it and then install it in order to continue.'
-        )
-    if thisversion != thatversion:
-        raise ImportError((
-            'hypothesis and hypothesislegacysupport must have exactly the '
-            'same version, but you have hypothesis==%s installed and '
-            'hypothesislegacysupport==%s installed. Please replace one '
-            'with a version compatible with the other.'
-        ) % (thisversion, thatversion))
+if sys.version_info[:2] <= (2, 6):
+    raise ImportError(
+        'Hypothesis is not supported on Python versions before 2.7'
+    )
 
-    from hypothesislegacysupport import GzipFile, bit_length, sha1, \
-        b64encode, b64decode
 
-    _special_floats = {
-        float(u'inf'): Decimal(u'Infinity'),
-        float(u'-inf'): Decimal(u'-Infinity'),
-    }
+def bit_length(n):
+    return n.bit_length()
 
-    def float_to_decimal(f):
-        """Convert a floating point number to a Decimal with no loss of
-        information."""
-        if f in _special_floats:
-            return _special_floats[f]
-        elif math.isnan(f):
-            return Decimal(u'NaN')
-        n, d = f.as_integer_ratio()
-        numerator, denominator = Decimal(n), Decimal(d)
-        ctx = Context(prec=60)
-        result = ctx.divide(numerator, denominator)
-        while ctx.flags[Inexact]:
-            ctx.flags[Inexact] = False
-            ctx.prec *= 2
-            result = ctx.divide(numerator, denominator)
-        return result
-else:
-    from gzip import GzipFile
-    from hashlib import sha1
-    from base64 import b64encode, b64decode
 
-    def bit_length(n):
-        return n.bit_length()
-
-    def float_to_decimal(f):
-        return Decimal(f)
+def float_to_decimal(f):
+    return Decimal(f)
 
 
 if PY3:
@@ -146,11 +106,28 @@ def quiet_raise(exc):
 
     def zero_byte_sequence(n):
         return bytes(n)
+
+    import struct
+
+    struct_pack = struct.pack
+    struct_unpack = struct.unpack
+
+    from time import monotonic as benchmark_time
 else:
     import struct
 
+    def struct_pack(*args):
+        return hbytes(struct.pack(*args))
+
+    if CAN_UNPACK_BYTE_ARRAY:
+        def struct_unpack(fmt, string):
+            return struct.unpack(fmt, string)
+    else:
+        def struct_unpack(fmt, string):
+            return struct.unpack(fmt, str(string))
+
     def zero_byte_sequence(n):
-        return b'\0' * n
+        return hbytes(b'\0' * n)
 
     def int_from_bytes(data):
         assert isinstance(data, bytearray)
@@ -183,7 +160,7 @@ else:
         return hbytes(result)
 
     def bytes_from_list(ls):
-        return bytes(bytearray(ls))
+        return hbytes(bytearray(ls))
 
     def to_bytes_sequence(ls):
         return bytearray(ls)
@@ -259,6 +236,8 @@ else:
     def quiet_raise(exc):
         raise exc
 
+    from time import time as benchmark_time
+
 
 def a_good_encoding():
     return 'utf-8'
@@ -281,50 +260,53 @@ def qualname(f):
     except AttributeError:
         return f.__name__
 
-FakeArgSpec = namedtuple(
-    'ArgSpec', ('args', 'varargs', 'keywords', 'defaults'))
 
+if PY2:
+    FullArgSpec = namedtuple('FullArgSpec', 'args, varargs, varkw, defaults, '
+                             'kwonlyargs, kwonlydefaults, annotations')
 
-def signature_argspec(f):
-    from inspect import signature, Parameter, _empty
-    try:
-        if NO_ARGSPEC:
-            sig = signature(f, follow_wrapped=False)
-        else:
-            sig = signature(f)
-    except ValueError:
-        raise TypeError('unsupported callable')
-    args = list(
-        k
-        for k, v in sig.parameters.items()
-        if v.kind in (
-            Parameter.POSITIONAL_ONLY, Parameter.POSITIONAL_OR_KEYWORD))
-    varargs = None
-    keywords = None
-    for k, v in sig.parameters.items():
-        if v.kind == Parameter.VAR_POSITIONAL:
-            varargs = k
-        elif v.kind == Parameter.VAR_KEYWORD:
-            keywords = k
-    defaults = []
-    for a in reversed(args):
-        default = sig.parameters[a].default
-        if default is _empty:
-            break
-        else:
-            defaults.append(default)
-    if defaults:
-        defaults = tuple(reversed(defaults))
-    else:
-        defaults = None
-    return FakeArgSpec(args, varargs, keywords, defaults)
-
-
-if NO_ARGSPEC:
-    getargspec = signature_argspec
-    ArgSpec = FakeArgSpec
+    def getfullargspec(func):
+        import inspect
+        args, varargs, varkw, defaults = inspect.getargspec(func)
+        return FullArgSpec(args, varargs, varkw, defaults, [], None,
+                           getattr(func, '__annotations__', {}))
 else:
-    from inspect import getargspec, ArgSpec
+    from inspect import getfullargspec, FullArgSpec
+
+    if sys.version_info[:2] == (3, 5):
+        # silence deprecation warnings on Python 3.5
+        # (un-deprecated in 3.6 to allow single-source 2/3 code like this)
+        def silence_warnings(func):
+            import warnings
+            import functools
+
+            @functools.wraps(func)
+            def inner(*args, **kwargs):
+                with warnings.catch_warnings():
+                    warnings.simplefilter('ignore', DeprecationWarning)
+                    return func(*args, **kwargs)
+            return inner
+
+        getfullargspec = silence_warnings(getfullargspec)
+
+
+if sys.version_info[:2] < (3, 6):
+    def get_type_hints(thing):
+        try:
+            spec = getfullargspec(thing)
+            return {
+                k: v for k, v in spec.annotations.items()
+                if k in (spec.args + spec.kwonlyargs) and isinstance(v, type)
+            }
+        except TypeError:
+            return {}
+else:
+    def get_type_hints(thing):
+        try:
+            import typing
+            return typing.get_type_hints(thing)
+        except TypeError:
+            return {}
 
 
 importlib_invalidate_caches = getattr(
@@ -419,10 +401,12 @@ class compatbytes(bytearray):
         raise ValueError('Value %r not in sequence %r' % (value, self))
 
     def __add__(self, value):
+        assert isinstance(value, compatbytes)
         return compatbytes(bytearray.__add__(self, value))
 
     def __radd__(self, value):
-        return compatbytes(bytearray.__radd__(self, value))
+        assert isinstance(value, compatbytes)
+        return compatbytes(bytearray.__add__(value, self))
 
     def __mul__(self, value):
         return compatbytes(bytearray.__mul__(self, value))
@@ -452,6 +436,7 @@ class compatbytes(bytearray):
     def __contains__(self, value):
         return any(v == value for v in self)
 
+
 if PY2:
     hbytes = compatbytes
     reasonable_byte_type = bytearray
@@ -462,6 +447,8 @@ else:
     string_types = (str,)
 
 
+EMPTY_BYTES = hbytes(b'')
+
 if PY2:
     def to_str(s):
         if isinstance(s, unicode):
@@ -483,6 +470,18 @@ def get_stream_enc(stream, default=None):
     return getattr(stream, 'encoding', None) or default
 
 
+def implements_iterator(it):
+    """Turn things with a __next__ attribute into iterators on Python 2."""
+    if PY2 and not hasattr(it, 'next') and hasattr(it, '__next__'):
+        it.next = it.__next__
+    return it
+
+
+if PY3:
+    FileNotFoundError = FileNotFoundError
+else:
+    FileNotFoundError = IOError
+
 # We need to know what sort of exception gets thrown when you try to write over
 # an existing file where you're not allowed to. This is rather less consistent
 # between versions than might be hoped.
@@ -490,10 +489,19 @@ if PY3:
     # Note: This only works on >= 3.3, but we only support >= 3.3 so that's
     # fine
     FileExistsError = FileExistsError
+
 elif WINDOWS:
     FileExistsError = WindowsError
+
 else:
     # This doesn't happen in this case: We're not on windows and don't support
     # the x flag because it's Python 2, so there are no places where this can
     # be thrown.
     FileExistsError = None
+
+
+if PY2:
+    def floor(x):
+        return int(math.floor(x))
+else:
+    floor = math.floor
